const CONNECTS_LEFT: [char; 3] = ['-', '7', 'J'];
const CONNECTS_RIGHT: [char; 3] = ['-', 'F', 'L'];
const CONNECTS_UP: [char; 3] = ['|', 'L', 'J'];
const CONNECTS_DOWN: [char; 3] = ['|', 'F', '7'];

fn main() {
    let input: Vec<Vec<char>> = include_str!("../../input.txt")
        .lines()
        .map(|line| line.chars().collect::<Vec<char>>())
        .collect();
    let mut distances: Vec<Vec<i32>> = input
        .clone()
        .iter()
        .map(|line| line
            .iter()
            .map(|_| i32::MAX).collect())
        .collect();
    let mut starting_pos = (0, 0);
    let mut start_adj: Vec<(usize, usize)> = vec![];

    // Find starting position
    'starting_pos: for (y, line) in input.iter().enumerate() {
        for (x, c) in line.iter().enumerate() {
            if c == &'S' {
                starting_pos = (x, y);
                
                if CONNECTS_RIGHT.contains(&input[y][x - 1]) {
                    start_adj.push((x - 1, y));
                }
                if CONNECTS_LEFT.contains(&input[y][x + 1]) {
                    start_adj.push((x + 1, y));
                }
                if CONNECTS_DOWN.contains(&input[y - 1][x]) {
                    start_adj.push((x, y - 1));
                }
                if CONNECTS_UP.contains(&input[y + 1][x]) {
                    start_adj.push((x, y + 1));
                }

                break 'starting_pos;
            }
        }
    }

    if start_adj.len() != 2 {
        panic!("There must be exactly 2 pipes connecting to the starting point");
    }

    distances[starting_pos.1][starting_pos.0] = 0;

    // Search in both directions
    let start_pos = start_adj.first().unwrap();
    let mut position = start_pos.clone();
    let mut distance = 1;
    let mut visited: Vec<Vec<bool>> = input
        .clone()
        .iter()
        .map(|line| line
            .iter()
            .map(|_| false).collect())
        .collect(); 

    visited[starting_pos.1][starting_pos.0] = true;

    loop {
        visited[position.1][position.0] = true;
        if distances[position.1][position.0] > distance {
            distances[position.1][position.0] = distance;
        }
        distance += 1;

        let mut next_nodes: Vec<(usize, usize)> = vec![];
        let current_char = input[position.1][position.0];
        if CONNECTS_LEFT.contains(&current_char) && CONNECTS_RIGHT.contains(&input[position.1][position.0 - 1]) {
            next_nodes.push((position.0 - 1, position.1));
        }
        if CONNECTS_RIGHT.contains(&current_char) && CONNECTS_LEFT.contains(&input[position.1][position.0 + 1]) {
            next_nodes.push((position.0 + 1, position.1));
        }
        if CONNECTS_UP.contains(&current_char) && CONNECTS_DOWN.contains(&input[position.1 - 1][position.0]) {
            next_nodes.push((position.0, position.1 - 1));
        }
        if CONNECTS_DOWN.contains(&current_char) && CONNECTS_UP.contains(&input[position.1 + 1][position.0]) {
            next_nodes.push((position.0, position.1 + 1));
        }

        if next_nodes.len() > 2 {
            println!("{:?}, {:?}", position, next_nodes);
            panic!("More than 2 adjacent pipes");
        }

        let next_node = next_nodes.iter().find(|node| !visited[node.1][node.0]);
        if let None = next_node {
            break;
        }

        position = *next_node.unwrap();
    }

    // this approach is entirely stolen from reddit :)
    let mut result = 0;
    for (y, line) in distances.iter().enumerate() {
        if y == distances.len() - 1 {
            break;
        }

        let mut capturing = false;
        for (x, d) in line.iter().enumerate() {
            if d == &i32::MAX {
                if capturing {
                    result += 1;
                    print!("*");
                } else {
                    print!(".");
                }
            } else if distances[y + 1][x] == d - 1 {
                capturing = false;
                print!("^");
            } else if distances[y + 1][x] == d + 1 {
                capturing = true;
                print!("v");
            } else {
                print!("-");
            }
        }

        print!("\n");
    }

    println!("Result: {result}");
}
