#[derive(Copy, Clone, Debug)]
struct Pair {
    first: i64,
    last: i64,
}

impl Pair {
    fn increment(&self, difference: i64) -> Pair {
        Pair {
            first: self.first + difference,
            last: self.last + difference,
        }
    }

    fn is_same(&self, other: &Pair) -> bool {
        self.first == other.first && self.last == other.last
    }
}

fn main() {
    let input = include_str!("../../input.txt").trim_end();
    let mut segments: Vec<&str> = input.split("\n\n").collect::<Vec<&str>>();
    let mut numbers: Vec<Pair> = Vec::new();
    let mut input_numbers: Vec<i64> = segments
        .remove(0)
        .replace("seeds: ", "")
        .split(" ")
        .map(|num| num.parse::<i64>().unwrap())
        .collect();

    while !input_numbers.is_empty() {
        let first = input_numbers.remove(0);
        let last = first + input_numbers.remove(0) - 1;

        numbers.push(Pair {
            first: first,
            last: last,
        });
    }

    for segment in segments {
        let mut new_numbers: Vec<Pair> = Vec::new();
        let mut lines = segment.trim().lines();
        lines.next();

        for line in lines {
            let values = line
                .split(" ")
                .map(|num| num.parse::<i64>().unwrap())
                .collect::<Vec<i64>>();

            let dest_start = values[0];
            let source_start = values[1];
            let range_len = values[2];
            let source_end = source_start + range_len - 1;

            for item in numbers.clone() {
                // Item is not in range
                if (item.first < source_start && item.last < source_start) || (item.first > source_end && item.last > source_end) {
                    continue;
                }

                // Item is completely within range
                if item.first >= source_start && item.last <= source_end {
                    new_numbers.push(item.increment(dest_start - source_start));
                    numbers.retain(|i| !i.is_same(&item));
                    continue;
                }

                // Item is overflowing to the left
                if item.first < source_start && item.last <= source_end {
                    new_numbers.push(Pair {
                        first: source_start,
                        last: item.last,
                    }.increment(dest_start - source_start));
                    numbers.push(Pair {
                        first: item.first,
                        last: source_start - 1,
                    });
                    numbers.retain(|i| !i.is_same(&item));
                    continue;
                }

                // Item is overflowing to the right
                if item.first >= source_start && item.last > source_end {
                    new_numbers.push(Pair {
                        first: item.first,
                        last: source_end,
                    }.increment(dest_start - source_start));
                    numbers.push(Pair {
                        first: source_end + 1,
                        last: item.last,
                    });
                    numbers.retain(|i| !i.is_same(&item));
                    continue;
                }

                // Item is overflowing on both sides
                if source_start > item.first && source_end < item.last {
                    new_numbers.push(Pair {
                        first: source_start,
                        last: source_end,
                    }.increment(dest_start - source_start));
                    numbers.push(Pair {
                        first: item.first,
                        last: source_start - 1,
                    });
                    numbers.push(Pair {
                        first: source_end + 1,
                        last: item.last,
                    });
                    numbers.retain(|i| !i.is_same(&item));
                    continue;
                }
            }
        }

        for num in &new_numbers {
            numbers.push(*num);
        }
    }

    numbers.sort_by(|a, b| a.first.cmp(&b.first));
    println!("Result: {}", &numbers[0].first);
}
